
# q-expansion of newform 912.3.o.b, downloaded from the LMFDB on 10 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 912
weight = 3
poly_data = [25, -5, -4, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[-15, -4, 4, 1], 10], [[5, 9, 1, -1], 5], [[25, 2, -2, -3], 10]]

hecke_ring_character_values = [[799, [1, 0, 0, 0]], [229, [1, 0, 0, 0]], [305, [1, 0, 0, 0]], [97, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, -1, 0, 0], [-2, 0, -1, 0], [-8, 0, -1, 0], [4, 0, -1, 0], [0, 4, 0, -2], [-4, 0, 5, 0], [1, -4, -2, -8], [-12, 0, -2, 0], [0, -8, 0, 4], [0, 4, 0, 10], [0, -28, 0, 6], [0, 12, 0, 8], [-16, 0, 15, 0], [-26, 0, 9, 0], [0, 36, 0, 12], [0, -28, 0, -16], [-16, 0, -3, 0], [0, 32, 0, 52], [0, 20, 0, -24], [24, 0, -3, 0], [0, -40, 0, -14], [-30, 0, 28, 0], [0, -4, 0, -48], [0, 52, 0, 16], [108, 0, 10, 0], [0, 80, 0, 38], [0, 32, 0, -36], [0, 40, 0, 38], [0, -80, 0, -44], [0, 32, 0, -30], [-12, 0, 47, 0], [-144, 0, 17, 0], [12, 0, -13, 0], [-170, 0, 23, 0], [0, -20, 0, -34], [38, 0, 28, 0], [-26, 0, 16, 0], [0, -136, 0, -44], [0, 0, 0, -8], [0, 8, 0, -20], [0, -48, 0, -18], [114, 0, 29, 0], [0, -88, 0, 32], [296, 0, -10, 0], [172, 0, -29, 0], [0, -20, 0, 80], [0, -84, 0, -70], [0, -180, 0, 32], [40, 0, -67, 0], [-128, 0, -23, 0], [166, 0, -55, 0], [0, 140, 0, 44], [-128, 0, 19, 0], [0, -48, 0, 44], [270, 0, -55, 0], [0, -180, 0, -132], [-26, 0, 40, 0], [304, 0, 45, 0], [0, -152, 0, -148], [4, 0, -13, 0], [0, -104, 0, -32], [0, 184, 0, 132], [-94, 0, 21, 0], [166, 0, -76, 0], [0, 196, 0, 20], [0, -112, 0, -20], [0, 36, 0, 136], [284, 0, 15, 0], [-516, 0, -23, 0], [-242, 0, 92, 0], [-194, 0, -71, 0], [438, 0, -40, 0], [0, 16, 0, 26], [0, -216, 0, 24], [0, 4, 0, -64], [234, 0, -61, 0], [-44, 0, 73, 0], [0, 208, 0, -28], [0, 164, 0, 200], [222, 0, -40, 0], [0, -196, 0, -182], [0, -172, 0, -252], [0, 80, 0, -12], [0, -120, 0, 110], [-404, 0, 63, 0], [0, -40, 0, -260], [124, 0, -39, 0], [-110, 0, 35, 0], [180, 0, -5, 0], [352, 0, 75, 0], [-640, 0, -54, 0], [0, 148, 0, -6], [166, 0, 144, 0], [16, 0, 87, 0], [684, 0, 54, 0], [0, 132, 0, 136], [0, -208, 0, -188], [0, 48, 0, 96], [404, 0, 137, 0], [0, -92, 0, -332], [10, 0, 27, 0], [0, 52, 0, 280], [0, -20, 0, -208], [-262, 0, 100, 0], [164, 0, 161, 0], [-408, 0, -5, 0], [-226, 0, -4, 0], [0, 496, 0, 44], [0, 92, 0, 248], [0, 204, 0, -122], [136, 0, 85, 0], [-348, 0, 61, 0], [-94, 0, 220, 0], [-16, 0, 191, 0], [0, -228, 0, 32], [-268, 0, -221, 0], [-206, 0, 85, 0], [474, 0, 59, 0], [0, 368, 0, 124], [0, 480, 0, -14], [0, -148, 0, 256], [0, 296, 0, 488], [0, -52, 0, 360], [-320, 0, -57, 0], [-600, 0, 102, 0], [26, 0, -80, 0], [30, 0, 33, 0], [68, 0, -77, 0], [194, 0, 232, 0], [172, 0, -213, 0], [0, -144, 0, 184], [0, 720, 0, 26], [-320, 0, 37, 0], [768, 0, 89, 0], [-320, 0, 93, 0], [0, -68, 0, 344], [0, -392, 0, 152], [0, -88, 0, 212], [892, 0, -3, 0], [0, -188, 0, 108], [-30, 0, -325, 0], [-248, 0, 215, 0], [0, 424, 0, 180], [0, 312, 0, -126], [0, -680, 0, -80], [618, 0, -64, 0], [0, -236, 0, -440], [1248, 0, -41, 0], [0, -20, 0, 396], [0, -16, 0, -182], [72, 0, 321, 0], [-1136, 0, 127, 0], [0, 180, 0, 296], [0, -524, 0, -244], [0, 352, 0, 356], [-110, 0, -140, 0], [-786, 0, 220, 0], [-208, 0, -19, 0], [0, 864, 0, 4], [-530, 0, 72, 0], [0, -328, 0, -564], [-758, 0, -52, 0], [0, 32, 0, -540], [0, -28, 0, -40], [0, 264, 0, 328], [0, -988, 0, -54], [-88, 0, -299, 0]]
